#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/stat.h>

#define BLOCK_SIZE 16
#define NB_FIB 46

int isFib(unsigned long n)
{
  static int fib[NB_FIB] = {0, 1};
  static int fillIndex = 1;
  int i;

  for (i = fillIndex; i < NB_FIB ; i++)
     fib[i + 1] = fib[i] + fib[i-1];
  fillIndex = i;
  for (i = 0; i < NB_FIB; i++) if (n == fib[i]) return 1;;
  return 0;
}

/*
  printBlock
		dumps a block of bytes in hex and ASCII
  Params:
		block = pointer to block to dump
		length = length of area to dump
		address = address in the file of the block
  Returns:
		-
  Changes:
    -
*/
void printBlock(const char *block, int length, long address)
{
  int i;

  printf("%8.8lX | ", address);
  for (i = 0; i < length; i++)
    if (isprint(block[i])) printf("%c", block[i]);
    else printf(".");
  while (i++ < BLOCK_SIZE) printf(" ");
  printf(" | ");
  for (i = 0; i < length; i++)
    printf("%2.2X ", (unsigned char)block[i]);
  while (i++ < BLOCK_SIZE) printf("   ");
  printf("\n");
}

int readBlock(FILE *f, char *block)
{
  // if (feof(f)) return -1;
  return fread(block, 1, BLOCK_SIZE, f);

}
int main(int argc, char*argv[])
{
    FILE *f;
    char block[BLOCK_SIZE];
    struct stat statBuf;
    int i, n;
    long address = 0;

    if (stat(argv[1], &statBuf) < 0)
    {
      perror(argv[1]);
      exit(1);
    }
    if (!S_ISREG(statBuf.st_mode))
    {
      fprintf(stderr, "%s dumps only regular files\n", argv[0]);
      exit(2);
    }
    printf("Dump of Fibonacci-numbered %d blocks of file %s [file is %ld bytes long]\n\n",
      BLOCK_SIZE, argv[1], (long)statBuf.st_size);
    printf(" Address          ASCII                     Hex                              \n");
    if (!(f = fopen(argv[1], "rb")))
    {
      perror(argv[1]);
      exit(1);
    }
    i = 0; // block count
    do
    {
      address = ftell(f);
      n = readBlock(f, block);
      if (n <= 0) break;

      if (isFib(i) && i >= 2) printBlock(block, n, address);
      i++;
      address = ftell(f);
    }
    while (n > 0);
    fclose(f);
    return 0;
}
